-- --------------
-- SimpleRune
-- by lieandswell
-- --------------


SimpleRune = {};
--	SIMPLERUNE = {}; 	is defined in the localization file, which must be loaded before this one. 

SIMPLERUNE.VERSION = "1.0.3";

SIMPLERUNE.RUNE_SPACING = 24;
SIMPLERUNE.FILL_SIZE = 18;
SIMPLERUNE.ARTTEXTURES = {
	[1] = "Interface\\AddOns\\SimpleRune\\Textures\\BloodRune",
	[2] = "Interface\\AddOns\\SimpleRune\\Textures\\BloodRune",
	[3] = "Interface\\AddOns\\SimpleRune\\Textures\\UnholyRune",
	[4] = "Interface\\AddOns\\SimpleRune\\Textures\\UnholyRune",
	[5] = "Interface\\AddOns\\SimpleRune\\Textures\\FrostRune",
	[6] = "Interface\\AddOns\\SimpleRune\\Textures\\FrostRune",
}

SIMPLERUNE.DEFAULTS = {
	Locked 		= false,
	Position	= { "BOTTOM", "BOTTOM", 0, 100 },
	RuneFrameBackgroundColor 	= { 0.05, 0.05, 0.05, 0.5 },
	RuneBackgroundColor			= { 0.05, 0.05, 0.05, 1.0 },
	RuneColor = {
		[1]		= { 0.7, 0.0, 0.0 },	-- blood
		[2]		= { 0.4, 0.8, 0.0 },	-- unholy
		[3]		= { 0.0, 0.4, 0.6 },	-- frost
		[4]		= { 0.6, 0.0, 0.7 },	-- death
	},
	Scale 			= 2,
	ShowBorder 		= false,
	ShowRuneArt 	= false,	
	UnusableAlpha 	= 0.4,
	Version 		= SIMPLERUNE.VERSION,
};

function SimpleRune.Test(stuff)
	if ( stuff ) then
		DEFAULT_CHAT_FRAME:AddMessage("SimpleRune test: "..stuff);
	else
		DEFAULT_CHAT_FRAME:AddMessage("SimpleRune test");
	end
end



-- ----------
-- MAIN FRAME
-- ----------

function SimpleRune.RuneFrame_OnLoad(self)
	local _, class = UnitClass("player");
	if ( class ~= "DEATHKNIGHT" ) then
		self:Hide();
		return;
	end

 	self:RegisterForDrag("LeftButton");
	self:RegisterEvent("VARIABLES_LOADED");
	self:RegisterEvent("PLAYER_ENTERING_WORLD");
	self:RegisterEvent("RUNE_POWER_UPDATE");
	self:RegisterEvent("RUNE_TYPE_UPDATE");

	self.background = getglobal(self:GetName().."Background");
	self.border = getglobal(self:GetName().."Border");
	self.resizeButton = getglobal(self:GetName().."ResizeButton");
	self.runes = {};
	for runeID = 1, 6 do
		self.runes[runeID] = getglobal(self:GetName().."Rune"..runeID);
	end
end

function SimpleRune.RuneFrame_OnEvent(self, event, ...)

	if ( event == "RUNE_POWER_UPDATE" ) then
		local runeID, usable = ...;
		local rune = self.runes[runeID];

		if ( not usable and rune ) then
			rune.fill:SetAlpha(SimpleRune_Settings.UnusableAlpha);
			rune.art:SetAlpha(SimpleRune_Settings.UnusableAlpha);
			rune:SetScript("OnUpdate", SimpleRune.Rune_OnUpdate);

			if ( rune.onDeck ) then
				local otherRuneID;
				if ( runeID % 2 == 0 ) then
					otherRuneID = runeID - 1;
				else
					otherRuneID = runeID + 1;
				end
				local otherStart, otherDuration, _ = GetRuneCooldown(otherRuneID);
				local otherRune = self.runes[otherRuneID];
				if ( not otherDuration ) then
					SimpleRune.SwitchPositions(rune, otherRune);
					rune.onDeck = nil;
					otherRune.onDeck = true;
				else
					local time = GetTime();
					local start, duration, _ = GetRuneCooldown(runeID);
					if ( start+duration-time > otherStart+otherDuration-time ) then
						SimpleRune.SwitchPositions(rune, otherRune);
						rune.onDeck = nil;
						otherRune.onDeck = true;
					end
				end
			end

		elseif ( usable and rune ) then
			rune.fill:SetAlpha(1);
			rune.art:SetAlpha(1);
		end

	elseif ( event == "RUNE_TYPE_UPDATE" ) then
		local runeID = ...;
		SimpleRune.Rune_UpdateType(self.runes[runeID], runeID);

	elseif ( event == "PLAYER_ENTERING_WORLD" ) then
		if ( _G["RuneFrame"] ) then
			_G["RuneFrame"]:Hide();
		end
		SimpleRune.Update();

	elseif ( event == 'VARIABLES_LOADED' ) then
		if ( not SimpleRune_Settings ) then
			SimpleRune_Settings = CopyTable(SIMPLERUNE.DEFAULTS);
		end
		SlashCmdList["SIMPLERUNE"] = SimpleRune.SlashCommand;
		SLASH_SIMPLERUNE1 = "/simplerune";
	end
end

function SimpleRune.Update()
	local runeFrame = getglobal("SimpleRune_RuneFrame");
	local settings = SimpleRune_Settings;

	runeFrame:ClearAllPoints();
	local point, relativePoint, xOfs, yOfs = unpack(settings.Position);
	runeFrame:SetPoint(point, UIParent, relativePoint, xOfs, yOfs);
	runeFrame:SetScale(settings.Scale);

	runeFrame.background:SetVertexColor(unpack(settings.RuneFrameBackgroundColor));
	if settings.ShowBorder then
		runeFrame.border:Show();
	else
		runeFrame.border:Hide();
	end

	for runeID = 1, 6 do
		SimpleRune.Rune_UpdateArt(runeFrame.runes[runeID], runeID);
		SimpleRune.Rune_UpdateType(runeFrame.runes[runeID], runeID);
	end

	if ( settings.Locked ) then
		runeFrame.resizeButton:Hide();
		runeFrame:EnableMouse(0);
		for runeID = 1, 6 do
			runeFrame.runes[runeID]:EnableMouse(0);
		end
	else
		runeFrame.resizeButton:Show();
		runeFrame:EnableMouse(1);
		for runeID = 1, 6 do
			runeFrame.runes[runeID]:EnableMouse(1);
		end
	end

end

function SimpleRune.SwitchPositions(rune1, rune2)
	local point, relativeTo, relativePoint, xOfs, yOfs = rune1:GetPoint();
	rune1:ClearAllPoints();
	rune1:SetPoint(rune2:GetPoint());
	rune2:ClearAllPoints();
	rune2:SetPoint(point, relativeTo, relativePoint, xOfs, yOfs);
end

function SimpleRune.OnEnter(self)
	local tooltip = getglobal("GameTooltip");
	GameTooltip_SetDefaultAnchor(tooltip, self);
	tooltip:AddLine("SimpleRune", HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b, 1);
	tooltip:AddLine(SIMPLERUNE.TOOLTIP_OPTIONS, NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b, 1);
	tooltip:Show();
end



-- -----
-- RUNES
-- -----

SIMPLERUNE.RUNE_POSITION = {
	[1]	= { point = "CENTER", x = -SIMPLERUNE.RUNE_SPACING, y = SIMPLERUNE.RUNE_SPACING/2}, 
	[2]	= { point = "CENTER", x = -SIMPLERUNE.RUNE_SPACING, y = -SIMPLERUNE.RUNE_SPACING/2}, 
	[5]	= { point = "CENTER", x = 0, y = SIMPLERUNE.RUNE_SPACING/2}, 
	[6]	= { point = "CENTER", x = 0, y = -SIMPLERUNE.RUNE_SPACING/2}, 
	[3]	= { point = "CENTER", x = SIMPLERUNE.RUNE_SPACING, y = SIMPLERUNE.RUNE_SPACING/2}, 
	[4]	= { point = "CENTER", x = SIMPLERUNE.RUNE_SPACING, y = -SIMPLERUNE.RUNE_SPACING/2}, 
};

function SimpleRune.Rune_OnLoad(self)
	self:RegisterForDrag("LeftButton");
	self.background = getglobal(self:GetName().."Background");
	self.fill = getglobal(self:GetName().."Fill");
	self.art = getglobal(self:GetName().."ArtTexture");

--	self.fill:SetTexCoordModifiesRect(true);

	self:ClearAllPoints();
	local coords = SIMPLERUNE.RUNE_POSITION[self:GetID()];
	self:SetPoint(coords.point, self:GetParent(), coords.point, coords.x, coords.y);
	if ( self:GetID() % 2 == 0 ) then
		self.onDeck = nil;
	else
		self.onDeck = true;
	end
end

function SimpleRune.Rune_UpdateArt(rune, runeID)
	local settings = SimpleRune_Settings;
	rune.background:SetVertexColor(unpack(settings.RuneBackgroundColor));
	rune.art:SetTexture(SIMPLERUNE.ARTTEXTURES[runeID]);
	if ( settings.ShowRuneArt ) then
		rune.art:Show();
	else
		rune.art:Hide();
	end
end

function SimpleRune.Rune_UpdateType(rune, runeID)
	rune.fill:SetVertexColor(unpack(SimpleRune_Settings.RuneColor[GetRuneType(runeID)]));
end

function SimpleRune.Rune_OnUpdate(self, elapsed)
	local start, duration, usable = GetRuneCooldown(self:GetID());
	if ( not usable ) then
		self.fill:SetHeight(SIMPLERUNE.FILL_SIZE * (GetTime() - start)/duration ); 
		self.fill:SetTexCoord(0, 1, (1 - ((GetTime() - start)/duration)), 1);
	else
		self.fill:SetHeight(SIMPLERUNE.FILL_SIZE);
		self.fill:SetTexCoord(0, 1, 0, 1);
		self:SetScript("OnUpdate", nil);
	end
end






