-- ---------------------
-- SimpleRune
-- by Nephthys of Hyjal
-- lieandswell@yahoo.com
-- ---------------------


function SimpleRune.SlashCommand(cmd)
	if ( cmd == SIMPLERUNE.CMD_RESET ) then
		SimpleRune.Reset();
	else 
		SimpleRune.LockToggle();
	end
end

-- -----------------------
-- INTERFACE OPTIONS PANEL
-- -----------------------

function SimpleRune.UIPanel_OnLoad(self)
	getglobal(self:GetName().."Version"):SetText(SIMPLERUNE.VERSION);
	getglobal(self:GetName().."SubText"):SetText(SIMPLERUNE.UIPANEL_SUBTEXT)
end

function SimpleRune.UIPanel_Update()
	local panelName = "InterfaceOptionsSimpleRunePanel";
	local panel = getglobal(panelName);
	local settings = SimpleRune_Settings;

	getglobal(panelName.."ShowRuneArtButton"):SetChecked(settings.ShowRuneArt);
	getglobal(panelName.."ShowBorderButton"):SetChecked(settings.ShowBorder);
	for runeType = 1, 4 do
		getglobal(panelName.."RuneColorButton"..runeType.."NormalTexture"):SetVertexColor(unpack(settings.RuneColor[runeType]));
	end
	getglobal(panelName.."RuneFrameBackgroundColorButton".."NormalTexture"):SetVertexColor(unpack(settings.RuneFrameBackgroundColor));
	getglobal(panelName.."RuneBackgroundColorButton".."NormalTexture"):SetVertexColor(unpack(settings.RuneBackgroundColor));

	local unusableAlphaSlider = getglobal(panelName.."UnusableAlphaSlider");
	unusableAlphaSlider:SetMinMaxValues(0, 1);
	unusableAlphaSlider:SetValue(settings.UnusableAlpha);
	SimpleRune.LockButton_Update();
end

function SimpleRune.CheckButton_OnClick(self, variable)
	if ( self:GetChecked() ) then
		SimpleRune_Settings[variable] = true;
	else
		SimpleRune_Settings[variable] = false;
	end
	SimpleRune.Update();
end

function SimpleRune.ChooseRuneColor(runeType)
	info = UIDropDownMenu_CreateInfo();
	info.r, info.g, info.b  = unpack(SimpleRune_Settings.RuneColor[runeType]);
	info.swatchFunc = SimpleRune.SetRuneColor;
	info.cancelFunc = SimpleRune.CancelRuneColor;
	info.extraInfo = runeType;
	OpenColorPicker(info);
end

function SimpleRune.SetRuneColor()
	local runeType = ColorPickerFrame.extraInfo;
	local r,g,b = ColorPickerFrame:GetColorRGB();
	SimpleRune_Settings.RuneColor[runeType] = {r, g, b};
	SimpleRune.Update();
	SimpleRune.UIPanel_Update();
end

function SimpleRune.CancelRuneColor(previousValues)
	if ( previousValues ) then
		local runeType = ColorPickerFrame.extraInfo;
		SimpleRune_Settings.RuneColor[runeType] = {previousValues.r, previousValues.g, previousValues.b};
		SimpleRune.Update();
		SimpleRune.UIPanel_Update();
	end
end

function SimpleRune.ChooseColor(variable)
	info = UIDropDownMenu_CreateInfo();
	info.r, info.g, info.b, info.opacity = unpack(SimpleRune_Settings[variable]);
	info.opacity = 1 - info.opacity;
	info.hasOpacity = true;
	info.opacityFunc = SimpleRune.SetOpacity;
	info.swatchFunc = SimpleRune.SetColor;
	info.cancelFunc = SimpleRune.CancelColor;
	info.extraInfo = variable;
	OpenColorPicker(info);
end

function SimpleRune.SetColor()
	local variable = ColorPickerFrame.extraInfo;
	local r,g,b = ColorPickerFrame:GetColorRGB();
	SimpleRune_Settings[variable][1] = r;
	SimpleRune_Settings[variable][2] = g;
	SimpleRune_Settings[variable][3] = b;
	SimpleRune.Update();
	SimpleRune.UIPanel_Update();
end

function SimpleRune.SetOpacity()
	local variable = ColorPickerFrame.extraInfo;
	SimpleRune_Settings[variable][4] = 1 - OpacitySliderFrame:GetValue();
	SimpleRune.Update();
	SimpleRune.UIPanel_Update();
end

function SimpleRune.CancelColor(previousValues)
	if ( previousValues ) then
		local variable = ColorPickerFrame.extraInfo;
		SimpleRune_Settings[variable] = {previousValues.r, previousValues.g, previousValues.b, previousValues.opacity};
		SimpleRune.Update();
		SimpleRune.UIPanel_Update();
	end
end

function SimpleRune.LockToggle()
	if ( SimpleRune_Settings["Locked"] ) then
		SimpleRune_Settings["Locked"] = false;
	else
		SimpleRune_Settings["Locked"] = true;
	end
	PlaySound("UChatScrollButton");
	SimpleRune.Update();
	SimpleRune.LockButton_Update();
end

function SimpleRune.LockButton_Update()
	local text = getglobal("InterfaceOptionsSimpleRunePanelLockButtonText");
	if ( SimpleRune_Settings["Locked"] ) then
		text:SetText(SIMPLERUNE.UIPANEL_UNLOCK);
	else
		text:SetText(SIMPLERUNE.UIPANEL_LOCK);
	end
end

function SimpleRune.Reset()
	SimpleRune_Settings = CopyTable(SIMPLERUNE.DEFAULTS);
	SimpleRune.Update();
	SimpleRune.UIPanel_Update();
end

function SimpleRune.Cancel()
	SimpleRune_Settings = CopyTable(SimpleRune_OldSettings);
	SimpleRune.Update();
end



-- -------------
-- RESIZE BUTTON
-- -------------

function SimpleRune.Resizebutton_OnEnter(self)
	local tooltip = getglobal("GameTooltip");
	GameTooltip_SetDefaultAnchor(tooltip, self);
	tooltip:AddLine(SIMPLERUNE.TOOLTIP_RESIZE, NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b, 1);
	tooltip:Show();
end

function SimpleRune.StartSizing(self, button)
	local runeFrame = self:GetParent();
	runeFrame.oldScale = runeFrame:GetScale();
	runeFrame.oldX = runeFrame:GetLeft();
	runeFrame.oldY = runeFrame:GetTop();
	runeFrame:ClearAllPoints();
	runeFrame:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", runeFrame.oldX, runeFrame.oldY);
	self.oldCursorX, self.oldCursorY = GetCursorPosition(UIParent);
	self:SetScript("OnUpdate", SimpleRune.Sizing_OnUpdate);
end

function SimpleRune.Sizing_OnUpdate(self)
	local uiScale = UIParent:GetScale();
	local cursorX, cursorY = GetCursorPosition(UIParent);
	local runeFrame = self:GetParent();

	-- calculate & set new scale
	local newYScale = runeFrame.oldScale * (cursorY/uiScale - runeFrame.oldY*runeFrame.oldScale) / (self.oldCursorY/uiScale - runeFrame.oldY*runeFrame.oldScale);
	local newXScale = runeFrame.oldScale * (cursorX/uiScale - runeFrame.oldX*runeFrame.oldScale) / (self.oldCursorX/uiScale - runeFrame.oldX*runeFrame.oldScale) ;
	local newScale = max(0.6, newYScale, newXScale);
	runeFrame:SetScale(newScale);

	-- set new frame coords to keep same on-screen position
	local newX = runeFrame.oldX * runeFrame.oldScale / newScale;
	local newY = runeFrame.oldY * runeFrame.oldScale / newScale;
	runeFrame:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", newX, newY);
end

function SimpleRune.StopSizing(self, button)
	self:SetScript("OnUpdate", nil)
	SimpleRune_Settings["Scale"] = self:GetParent():GetScale();
	SimpleRune.SavePosition(self:GetParent());
end

function SimpleRune.SavePosition(runeFrame)
	local point, _, relativePoint, xOfs, yOfs = runeFrame:GetPoint();
	SimpleRune_Settings["Position"] = {point, relativePoint, xOfs, yOfs};
end




